import ScenarioFunctions

from CvPythonExtensions import *
import CvUtil
import Popup as PyPopup
import PyHelpers
import CvScreenEnums
import CustomFunctions
import CvEspionageAdvisor

# globals
cf = CustomFunctions.CustomFunctions()
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer

class ScenarioFunctionsPOD( ScenarioFunctions.ScenarioFunctionsDefault ):
	def __init__(self):
		# Cannot use any member data because init won't be called on exit to main menu/open
		pass

	def getPirateWarning( self ):
		return gc.getGame().getScenarioCounter() & 3

	def changePirateWarning( self, value ):
		gc.getGame().changeScenarioCounter( value )

	def wasIlliansCapitalTaken( self ):
		return gc.getGame().getScenarioCounter() & 4

	def setIlliansCapitalTaken( self ):
		if not self.wasIlliansCapitalTaken():
			gc.getGame().changeScenarioCounter( 4 )

	def getPirateIsDead( self ):
		return (gc.getGame().getScenarioCounter() & 8) != 0

	def setPirateIsDead( self ):
		if not self.getPirateIsDead():
			gc.getGame().changeScenarioCounter( 8 )

	def isFirstACLevelReached( self ):
		return ((gc.getGame().getScenarioCounter() & 16) == 0) and CyGame().getGlobalCounter() > 28

	def setFirstACLevelReached( self ):
		if (gc.getGame().getScenarioCounter() & 16 == 0):
			gc.getGame().changeScenarioCounter( 16 )

	def isSecondACLevelReached( self ):
		return ((gc.getGame().getScenarioCounter() & 32) == 0) and CyGame().getGlobalCounter() > 48

	def setSecondACLevelReached( self ):
		if (gc.getGame().getScenarioCounter() & 32 == 0):
			gc.getGame().changeScenarioCounter( 32 )

	def isThirdACLevelReached( self ):
		return ((gc.getGame().getScenarioCounter() & 64) == 0) and CyGame().getGlobalCounter() > 62

	def setThirdACLevelReached( self ):
		if (gc.getGame().getScenarioCounter() & 64 == 0):
			gc.getGame().changeScenarioCounter( 64 )

	def isFourthACLevelReached( self ):
		return ((gc.getGame().getScenarioCounter() & 128) == 0) and CyGame().getGlobalCounter() > 94

	def setFourthACLevelReached( self ):
		if (gc.getGame().getScenarioCounter() & 128 == 0):
			gc.getGame().changeScenarioCounter( 128 )

	def isACIncrease( self ):
		return ((gc.getGame().getScenarioCounter() & 256) == 0)

	def toggleACIncrease( self ):
		if (gc.getGame().getScenarioCounter() & 256 == 0):
			gc.getGame().changeScenarioCounter( 256 )
		else:
			gc.getGame().changeScenarioCounter( -256 )

	def gameStart(self):
		gc.getGame().changeScenarioCounter( 0 )
		self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_INTRO",()), 'art/interface/popups/Prince of Darkness.dds')
		CyMap().plot(36,46).setMoveDisabledHuman(True) #Gates of Dis
		#Around Dis for flyers:
		for i in range(34,40):
			CyMap().plot(i,46).setMoveDisabledHuman(True)
		for i in range(47,52):
			CyMap().plot(34,i).setMoveDisabledHuman(True)
			CyMap().plot(39,i).setMoveDisabledHuman(True)

	def doTurn(self):
		if CyGame().getGameTurn() == 0:
			self.doTurnZeroPrinceOfDarkness()
		if self.getPirateWarning() > 0:
			gc.getGame().changeScenarioCounter( -1 )
			if self.getPirateWarning() == 0:
				self.warnAboutPiratesPrinceOfDarkness()
		if not self.isBabelAcquired() and not self.isBabelInfernal():
			self.checkBabelPrinceOfDarkness()
		if CyMap().plot(12,11).isPythonActive():
			self.checkFlyingCastlePrinceOfDarkness()
		if not CyMap().plot(67,50).isPythonActive():
			self.checkGuardianOfFire()
		if not gc.getPlayer(1).isAlive():
			for iLoopPlayer in range(gc.getMAX_PLAYERS()):
				pLoopPlayer = gc.getPlayer(iLoopPlayer)
				if pLoopPlayer.isHuman():
					gc.getGame().setWinner(pLoopPlayer.getTeam(), 2)
		self.doArmageddonCounterEffects()

	def onCityAcquired(self, iPreviousOwner, iNewOwner, pCity, bConquest, bTrade):
		#Increase Armageddon Counter every time a city is taken because cities can't be razed
		CyGame().changeGlobalCounter(1)
		pPlayer = gc.getPlayer(iNewOwner)
		if pPlayer.isHuman():
			if pCity.getName() == "Wise Men Isle":
				self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_1",()), 'art/interface/popups/WiseMen.dds')
				iUnit = gc.getInfoTypeForString('EQUIPMENT_HEALING_SALVE')
				newUnit = pPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
				for i in range(3):
					for j in range(5):
						gc.getMap().plot(48+i,26+j).setRevealed(pPlayer.getTeam(), true, true, -1)
				self.setWiseMenAcquired()
			elif pCity.getName() == CyTranslator().getText("TXT_KEY_CITY_ILLIANS_1",()) and not self.wasIlliansCapitalTaken():
				iEvent = gc.getInfoTypeForString('EVENTTRIGGER_WB_PRINCE_OF_DARKNESS_SCROLLS')
				triggerData = pPlayer.initTriggeredData(iEvent, true, -1, -1, -1, 0, -1, -1, -1, -1, -1)
				self.setIlliansCapitalTaken()
				self.changePirateWarning( 3 )
				for i in range(5):
					for j in range(3):
						gc.getMap().plot(53+i,1+j).setRevealed(pPlayer.getTeam(), true, false, -1)
			elif pCity.getName() == "Alexos by the Sea":
				if pPlayer.isHuman() and not self.isAlexosAcquired():
					CyGame().changeGlobalCounter(-5)
					for i in range(5):
						for j in range(5):
							gc.getMap().plot(9+i,9+j).setRevealed(pPlayer.getTeam(), true, true, -1)
					self.setAlexosAcquired()
					if self.getPirateIsDead():
						self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_ALEXOS",()), 'art/interface/popups/Alexos.dds')
					else:
						self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_ALEXOS_WARN_PIRATE",()), 'art/interface/popups/Alexos.dds')
		if pCity.getName() == "Dis":
			for iLoopPlayer in range(gc.getMAX_PLAYERS()):
				pLoopPlayer = gc.getPlayer(iLoopPlayer)
				if pLoopPlayer.isHuman():
					gc.getGame().setWinner(pLoopPlayer.getTeam(), 2)
		if pCity.getName() == "Babel":
			self.setBabelAcquired()
			pPlayer = gc.getPlayer(iNewOwner)
			if (iNewOwner == 1 or iNewOwner == 6) and not self.isBabelInfernal(): #Hyborem or Sheaim
				eTeam = gc.getTeam(pPlayer.getTeam())
				iTech = gc.getInfoTypeForString('TECH_NECROMANCY')
				if eTeam.isHasTech(iTech):
					iTech = gc.getInfoTypeForString('TECH_WRITING')
					CyGame().changeGlobalCounter(1)
				if eTeam.isHasTech(iTech):
					iTech = gc.getInfoTypeForString('TECH_SORCERY')
					CyGame().changeGlobalCounter(1)
				if eTeam.isHasTech(iTech):
					iTech = gc.getInfoTypeForString('TECH_ARCANE_LORE')
					CyGame().changeGlobalCounter(1)
				if eTeam.isHasTech(iTech):
					iTech = gc.getInfoTypeForString('TECH_STRENGTH_OF_WILL')
					CyGame().changeGlobalCounter(1)
				if not eTeam.isHasTech(iTech):
					eTeam.setHasTech(iTech, True, 1, False, False)
				self.setBabelInfernal()
				CyGame().changeGlobalCounter(2)
		if not CyMap().plot(67,50).isPythonActive() and not CyMap().plot( pCity.getX(), pCity.getY() ).isPythonActive():
			self.angerGuardianOfFire(pCity.getName(),pPlayer.isHuman())

	def onImprovementDestroyed(self, iImprovement, iOwner, iX, iY):
		if iImprovement == gc.getInfoTypeForString('IMPROVEMENT_JUNGLE_ALTAR'):
			if (iX == 50 and iY == 30):
				CyMap().plot(50,30).setPythonActive(False)
				self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_3",()), 'art/interface/popups/WizardSaved.dds')
				for iLoopPlayer in range(gc.getMAX_PLAYERS()):
					pLoopPlayer = gc.getPlayer(iLoopPlayer)
					if pLoopPlayer.isHuman():
						iUnit = gc.getInfoTypeForString('UNIT_WIZARD')
						newUnit = pLoopPlayer.initUnit(iUnit, 50, 30, UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
						newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_HERO'), True)
						newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_MOBILITY1'), True)
						newUnit.setName('Ilmarinen')
					for i in range(5):
						for j in range(5):
							gc.getMap().plot(9+i,45+j).setRevealed(pLoopPlayer.getTeam(), true, true, -1)

	def onUnitKilled(self, pUnit, iAttacker):
		if pUnit.getUnitType() == gc.getInfoTypeForString('UNIT_SONS_OF_ASENA'):
			self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_2",()), 'art/interface/popups/DarkGuardianKilled.dds')
		elif pUnit.getUnitType() == gc.getInfoTypeForString('UNIT_FRIGATE') and pUnit.getNameNoDesc() == "Captain WhiteHeart":
			pPlayer = gc.getPlayer(iAttacker)
			if pPlayer.isHuman():
				self.onWhiteHeartKilled()
			else:
				self.onWhiteHeartRespawn()

	def onMoveWarningPost(self, pCaster, pPlot):
		if pCaster.isHuman() and pPlot.getX() == 11 and pPlot.getY() == 47:
			self.onMoveKhelavasterPrinceOfDarkness(pCaster)
		elif pCaster.isHuman() and pPlot.getX() == 9 and (pPlot.getY() == 11 or pPlot.getY() == 12 or pPlot.getY() == 13) and CyMap().plot(9,11).isPythonActive() and CyMap().plot(12,11).isPythonActive():
			self.onNearFloatingCastleWest()
		elif pCaster.isHuman() and ((pPlot.getX() == 15 and pPlot.getY() == 13) or (pPlot.getX() == 16 or pPlot.getY() == 13) or (pPlot.getX() == 16 or pPlot.getY() == 12)) and CyMap().plot(9,11).isPythonActive() and CyMap().plot(12,11).isPythonActive():
			self.onNearFloatingCastleEast()
		elif not CyMap().plot(12,11).isPythonActive() and pCaster.isHuman() and pPlot.getX() == 67 and pPlot.getY() == 50 and CyMap().plot(67,50).isPythonActive():
			self.onIsleOfDeceptionPrinceOfDarkness()

	def doTurnZeroPrinceOfDarkness(self):
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):
			pLoopPlayer = gc.getPlayer(iLoopPlayer)
			if pLoopPlayer.isHuman():
				pCity = gc.getPlayer(iLoopPlayer).getCapitalCity()
				iUnit = gc.getInfoTypeForString('UNIT_CHAMPION')
				newUnit = pLoopPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_HERO'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COURAGE'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_IRON_WEAPONS'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_MOBILITY1'), True)
				newUnit.setName('Lord Vladimir')
				for pyCity in PyPlayer(iLoopPlayer).getCityList() :
					pCity = pyCity.GetCy()
					if gc.getMap().plot(pCity.getX(),pCity.getY()).isCoastalLand():
						iUnit = gc.getInfoTypeForString('UNIT_QUEEN_OF_THE_LINE')
						newUnit = pLoopPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
						newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_HEROIC_STRENGTH'), True)
						newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_HEROIC_DEFENSE'), True)
						newUnit.setName('Ocean Queen')
						break
				for i in range(5):
					for j in range(5):
						gc.getMap().plot(28+i,18+j).setRevealed(pLoopPlayer.getTeam(), true, false, -1)
		self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_0",()), 'art/interface/popups/LordVladimir.dds')

	def checkBabelPrinceOfDarkness(self):
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):
			pLoopPlayer = gc.getPlayer(iLoopPlayer)
			if pLoopPlayer.isHuman():
				pPlayer = pLoopPlayer
		for i in range(3):
			for j in range(3):
				if not self.isBabelAcquired():
					self.checkBabelPlotPrinceOfDarkness(41+i,4+j)
					if self.isBabelAcquired():
						self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_WELCOME_BABEL",()), 'art/interface/popups/POD-WelcomeBabel.dds')
						pPlot = CyMap().plot(41,4)
						iUnit = gc.getInfoTypeForString('UNIT_CROSSBOWMAN')
						newUnit = pPlayer.initUnit(iUnit, 41+i, 4+j, UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
						for i in range(pPlot.getNumUnits(), -1, -1):
							pUnit = pPlot.getUnit(i)
							pUnit.kill(True,0)
						#The units are not killed if I create a unit in the city!!
						pCity = CyMap().plot(41,4).getPlotCity()
						pPlayer.acquireCity(pCity,false,false)

	def checkBabelPlotPrinceOfDarkness(self,x,y):
		pPlot = CyMap().plot(x,y)
		for i in range(pPlot.getNumUnits(), -1, -1):
			pUnit = pPlot.getUnit(i)
			pPlayer = gc.getPlayer(pUnit.getOwner())
			if None != pPlayer and pPlayer.isHuman():
				self.setBabelAcquired()

	def warnAboutPiratesPrinceOfDarkness(self):
		self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_PIRATE",()), 'art/interface/popups/POD-Pirate.dds')
		pPlayer = gc.getPlayer(1) #Hyborem
		serpent = pPlayer.initUnit(gc.getInfoTypeForString('UNIT_SEA_SERPENT'), 50, 10, UnitAITypes.UNITAI_ATTACK_SEA, DirectionTypes.DIRECTION_SOUTH)
		serpent = pPlayer.initUnit(gc.getInfoTypeForString('UNIT_SEA_SERPENT'), 49, 9, UnitAITypes.UNITAI_ATTACK_SEA, DirectionTypes.DIRECTION_SOUTH)
		serpent = pPlayer.initUnit(gc.getInfoTypeForString('UNIT_SEA_SERPENT'), 49, 8, UnitAITypes.UNITAI_ATTACK_SEA, DirectionTypes.DIRECTION_SOUTH)
		serpent = pPlayer.initUnit(gc.getInfoTypeForString('UNIT_SEA_SERPENT'), 50, 7, UnitAITypes.UNITAI_ATTACK_SEA, DirectionTypes.DIRECTION_SOUTH)
		self.onWhiteHeartRespawn()
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):
			pLoopPlayer = gc.getPlayer(iLoopPlayer)
			if pLoopPlayer.isHuman():
				for i in range(5):
					for j in range(3):
						gc.getMap().plot(39+i,2+j).setRevealed(pLoopPlayer.getTeam(), true, false, -1)

	def onWhiteHeartRespawn(self):
		#If the pirate is not killed by human, resurrect it.
		pPlot = cf.findClearPlot(-1, gc.getMap().plot(50,8))
		pPlayer = gc.getPlayer(1) #Hyborem
		iUnit = gc.getInfoTypeForString('UNIT_FRIGATE')
		newUnit = pPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_SEA, DirectionTypes.DIRECTION_SOUTH)
		newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_DEMON'), True)
		newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BUCCANEERS'), True)
		newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COMBAT1'), True)
		newUnit.setName('Captain WhiteHeart')

	def onWhiteHeartKilled(self):
			self.setPirateIsDead()
			self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_PIRATE_DEAD",()), 'art/interface/popups/POD-Pirate.dds')
			for iLoopPlayer in range(gc.getMAX_PLAYERS()):
				pLoopPlayer = gc.getPlayer(iLoopPlayer)
				if pLoopPlayer.isHuman():
					for i in range(5):
						for j in range(5):
							gc.getMap().plot(39+i,2+j).setRevealed(pLoopPlayer.getTeam(), true, true, -1)
					eTeam = gc.getTeam(pLoopPlayer.getTeam())
					iTech = gc.getInfoTypeForString('TECH_FISHING')
					if eTeam.isHasTech(iTech):
						iTech = gc.getInfoTypeForString('TECH_SAILING')
					if eTeam.isHasTech(iTech):
						iTech = gc.getInfoTypeForString('TECH_OPTICS')
					if eTeam.isHasTech(iTech):
						iTech = gc.getInfoTypeForString('TECH_WRITING')
					if eTeam.isHasTech(iTech):
						iTech = gc.getInfoTypeForString('TECH_MATHEMATICS')
					if eTeam.isHasTech(iTech):
						iTech = gc.getInfoTypeForString('TECH_ASTRONOMY')
					if not eTeam.isHasTech(iTech):
						eTeam.setHasTech(iTech, True, iLoopPlayer, False, True)
					CyGame().changeGlobalCounter(-5)

	def onMoveKhelavasterPrinceOfDarkness(self,pCaster):
		CyMap().plot(11,47).setPythonActive( False )
		self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_4",()), 'art/interface/popups/Khelavaster.dds')
		iUnit = gc.getInfoTypeForString('EQUIPMENT_GOLDEN_HAMMER')
		iPlayer = pCaster.getOwner()
		pPlayer = gc.getPlayer(iPlayer)
		newUnit = pPlayer.initUnit(iUnit, 11, 47, UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
		for i in range(5):
			for j in range(5):
				gc.getMap().plot(78+i,4+j).setRevealed(pPlayer.getTeam(), true, true, -1)

	def onNearFloatingCastleWest(self):
		self.onNearFloatingCastle(10,11)

	def onNearFloatingCastleEast(self):
		self.onNearFloatingCastle(15,12)

	def onNearFloatingCastle(self,x,y):
		CyMap().plot(9,11).setPythonActive(False)
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):
			pLoopPlayer = gc.getPlayer(iLoopPlayer)
			if pLoopPlayer.isHuman():
				self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_FLOATING_CASTLE_SHIP",()), 'art/interface/popups/FloatingCastleShip.dds')
				iUnit = gc.getInfoTypeForString('UNIT_AIRSHIP')
				newUnit = pLoopPlayer.initUnit(iUnit, x, y, UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
				newUnit.setLevel( 3 )
				newUnit.setExperience( 8, 100 )
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COMBAT1'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_COMBAT2'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_AIR2'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_STRONG'), True)

	def checkFlyingCastlePrinceOfDarkness(self):
		pPlot = CyMap().plot(12,11)
		for i in range(pPlot.getNumUnits(), -1, -1):
			pUnit = pPlot.getUnit(i)
			pPlayer = gc.getPlayer(pUnit.getOwner())
			if None != pPlayer and pPlayer.isHuman():
				CyMap().plot(12,11).setPythonActive(False)
				self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_FLOATING_CASTLE",()), 'art/interface/popups/FloatingCastle.dds')
				for i in range(3):
					for j in range(3):
						gc.getMap().plot(66+i,49+j).setRevealed(pPlayer.getTeam(), true, false, -1)
				break

	def onIsleOfDeceptionPrinceOfDarkness(self):
		#Must give a weapon to the player, that can destroy the Gates of Dis...
		#Easiest is to create a dedicated unit for that...
		CyMap().plot(67,50).setPythonActive(False)
		fireHeartPlayer = []
		hawk = None
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):
			pLoopPlayer = gc.getPlayer(iLoopPlayer)
			if pLoopPlayer.isHuman():
				iUnit = gc.getInfoTypeForString('UNIT_RANGER')
				newUnit = pLoopPlayer.initUnit(iUnit, 67, 50, UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_FIRE2'), True)
				newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_UNDEAD'), True)
				newUnit.setName('Guardian of the Fire')
				iUnit = gc.getInfoTypeForString('UNIT_HAWK')
				hawk = pLoopPlayer.initUnit(iUnit, 67, 50, UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
				hawk.setName('Heart of the Fire')
				#hawk.setTransportUnit( newUnit ) Carrier can't be loaded onboard...
			elif iLoopPlayer != 1 and iLoopPlayer != 6:
				pPlayer = gc.getPlayer(iLoopPlayer)
				if pPlayer.isAlive():
					fireHeartPlayer.append(iLoopPlayer)
		iFireHeartPlayer = fireHeartPlayer[gc.getGame().getSorenRandNum(len(fireHeartPlayer), "Fire Heart")]
		pCity = gc.getPlayer(iFireHeartPlayer).getCapitalCity()
		hawk.setScenarioCounter( iFireHeartPlayer + 10 * pCity.getID() )
		self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_ISLE_OF_DECEPTION",(pCity.getName(),)),'art/interface/popups/IsleOfDeception.dds')

	def getGoalTag(self, pPlayer):
		szBuffer = u"<font=2>"
		if not self.isWiseMenAcquired():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_WISEMEN_ISLE", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		if CyMap().plot(50,30).isPythonActive():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_DARK_DUNGEON", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		if CyMap().plot(11,47).isPythonActive():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_KHELAVASTER", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		if not self.wasIlliansCapitalTaken():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_ILLIANS", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		if self.getPirateWarning() == 0 and not self.getPirateIsDead():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_PIRATE", (), gc.getInfoTypeForString("COLOR_WHITE")) + u"<br/>"
		if not self.isAlexosAcquired():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_ALEXOS", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		if CyMap().plot(12,11).isPythonActive():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_CASTLE", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		if CyMap().plot(67,50).isPythonActive():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_ISLE_OF_DECEPTION", (), gc.getInfoTypeForString("COLOR_RED"))
			return szBuffer + "</font>"
		hawk = self.find( "Heart of the Fire" )
		guardian = self.find( "Guardian of the Fire" )
		if hawk!= None and guardian != None:
			iPlayer, iCity = self.getPlayerAndCity( hawk.getScenarioCounter() )
			pPlayer = gc.getPlayer(iPlayer)
			pCity = pPlayer.getCity(iCity)
			if pCity != None:
				szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_BURST", (pCity.getName(),), gc.getInfoTypeForString("COLOR_WHITE")) + u"<br/>"
		if CyMap().plot(36,46).isMoveDisabledHuman():
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_GATES_OF_DIS", (), gc.getInfoTypeForString("COLOR_RED"))
		else:
			szBuffer = szBuffer + CyTranslator().getColorText("TXT_KEY_WB_PRINCE_OF_DARKNESS_GOAL_BEAT_HYBOREM", (), gc.getInfoTypeForString("COLOR_RED"))
		return szBuffer + "</font>"

	#City acquired checks do not use the city plot because I temporarily used city plot to know the birth city of Guardian of Fire, in order to avoid silly conflicts.
	def isWiseMenAcquired(self):
		return not CyMap().plot(30,21).isPythonActive()
	def setWiseMenAcquired(self):
		CyMap().plot(30,21).setPythonActive(False)

	def setBabelAcquired(self):
		CyMap().plot(42,4).setPythonActive(False)
	def isBabelAcquired(self):
		return not CyMap().plot(42,4).isPythonActive()

	def isBabelInfernal(self):
		return not CyMap().plot(41,5).isPythonActive()
	def setBabelInfernal(self):
		CyMap().plot(41,5).setPythonActive(False)

	def setAlexosAcquired(self):
		CyMap().plot(54,2).setPythonActive(False)
	def isAlexosAcquired(self):
		return not CyMap().plot(54,2).isPythonActive()

	def angerGuardianOfFire(self,city,human):
		guardianUnit = self.find( "Guardian of the Fire" )
		heartUnit = self.find( "Heart of the Fire" )
		if( guardianUnit != None ):
			if human:
				self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_ANGER_GUARDIAN_OF_FIRE_HUMAN",(city,)), 'art/interface/popups/BurstGuardianOfFire.dds')
			else:
				self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_ANGER_GUARDIAN_OF_FIRE_AI",(city,)), 'art/interface/popups/BurstGuardianOfFire.dds')
			self.burstGuardian(guardianUnit,heartUnit)

	def find(self,unitName):
		for iLoopPlayer in range(gc.getMAX_PLAYERS()):
			pLoopPlayer = gc.getPlayer(iLoopPlayer)
			if pLoopPlayer.isHuman():
				(loopUnit, iter) = pLoopPlayer.firstUnit(false)
				while( loopUnit ):
					if loopUnit.getNameNoDesc() == unitName:
						return loopUnit
					(loopUnit, iter) = pLoopPlayer.nextUnit(iter, false)
		return None

	def getPlayerAndCity( self, counter ):
		player = counter%10
		city = ( counter - player )/10
		return player, city

	def checkGuardianOfFire(self):
		heart = self.find( "Heart of the Fire" )
		guardianUnit = self.find( "Guardian of the Fire" )
		if None != heart and None != guardianUnit:
			plot = heart.plot()
			player, city = self.getPlayerAndCity( heart.getScenarioCounter() )
			if plot.isCity() and plot.getPlotCity().getID() == city and plot.getOwner() == player:
				point = guardianUnit.plot().getPoint()
				CyEngine().triggerEffect(gc.getInfoTypeForString('EFFECT_ARTILLERY_SHELL_EXPLODE'),point)
				CyAudioGame().Play3DSound("AS3D_UN_GRENADE_EXPLODE",point.x,point.y,point.z)
				self.burstGuardian(guardianUnit,heart)

	def burstGuardian(self,guardianUnit,heartUnit):
		heartUnit.kill(True,0)
		plot = guardianUnit.plot()
		for x in range( plot.getX() - 1, plot.getX() + 2 ):
			for y in range( plot.getY() - 1, plot.getY() + 2 ):
				pPlot = CyMap().plot(x,y)
				for i in range(pPlot.getNumUnits(), -1, -1):
					pUnit = pPlot.getUnit(i)
					pUnit.kill(True,0)
				pPlot.setImprovementType(-1)
				if x == 36 and y == 46: #Gates of Dis
					pPlot.setMoveDisabledHuman(False)
		if CyMap().plot(36,46).isMoveDisabledHuman():
			gc.getGame().setWinner(gc.getPlayer(1).getTeam(), 2)

	def onVictory(self, iPlayer, iVictory):
		self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_VICTORY",()), 'art/interface/popups/Prince of Darkness.dds')

	def onTechAcquired(self, iTechType, iTeam, iPlayer, bAnnounce):
		if (iPlayer == 1): #Hyborem
			if self.isACIncrease():
				#CyGame().changeGlobalCounter(1) #Rank up the Armageddon Counter once every other tech!
				pass #so maybe that's a bit overdone...
			self.toggleACIncrease()

	def doArmageddonCounterEffects(self):
		pHyboremPlayer = gc.getPlayer(1)
		if self.isFirstACLevelReached():
			self.setFirstACLevelReached()
			#Create a few units for Hyborem.
			#Near Kalocly
			iUnit = gc.getInfoTypeForString('UNIT_DISEASED_CORPSE')
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(28,31))
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(31,30))
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			#Near Spilan
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(34,26))
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			iUnit = gc.getInfoTypeForString('UNIT_MANTICORE')
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_WEAK'), True)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_WITHERED'), True)
			#Seven Pines
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(6,42))
			iUnit = gc.getInfoTypeForString('UNIT_TREANT')
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			#Ring of Carcer
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(64,6))
			iUnit = gc.getInfoTypeForString('UNIT_DISEASED_CORPSE')
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			iUnit = gc.getInfoTypeForString('UNIT_MINOTAUR')
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK, DirectionTypes.DIRECTION_SOUTH)
			#Zbolub
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(9,16))
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_BRONZE_WEAPONS'), True)
			iUnit = gc.getInfoTypeForString('UNIT_MANTICORE')
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_WEAK'), True)
			newUnit.setHasPromotion(gc.getInfoTypeForString('PROMOTION_WITHERED'), True)
			#Lothenar
			pPlot = cf.findClearPlot(-1, gc.getMap().plot(79,46))
			newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_ATTACK_CITY, DirectionTypes.DIRECTION_SOUTH)
		if self.isSecondACLevelReached():
			self.setSecondACLevelReached()
			#Gift Veil prophets to everybody Savants
			self.addPopupWB(CyTranslator().getText("TXT_KEY_WB_PRINCE_OF_DARKNESS_SAVANTS",()), 'art/interface/popups/Savants.dds')
			iUnit = gc.getInfoTypeForString('UNIT_DISCIPLE_THE_ASHEN_VEIL')
			for iLoopPlayer in range(gc.getMAX_PLAYERS()):
				pLoopPlayer = gc.getPlayer(iLoopPlayer)
				if pLoopPlayer.isHuman():
					pCity = pLoopPlayer.getCapitalCity()
					newUnit = pLoopPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
					newUnit = pLoopPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), UnitAITypes.NO_UNITAI, DirectionTypes.DIRECTION_SOUTH)
				elif pLoopPlayer.isAlive():
					pCity = pLoopPlayer.getCapitalCity()
					if iLoopPlayer < 10:
						newUnit = pLoopPlayer.initUnit(iUnit, pCity.getX(), pCity.getY(), UnitAITypes.UNITAI_MISSIONARY, DirectionTypes.DIRECTION_SOUTH)
		if self.isThirdACLevelReached():
			self.setThirdACLevelReached()
			#Commented away because AC skyrockets and may be related to a ctd
			#Gift most remaining barbarian cities to Hyborem
			#for pyCity in PyPlayer(gc.getBARBARIAN_PLAYER()).getCityList() :
				#if 5 < gc.getGame().getSorenRandNum( 10, "convert to Hyborem" ):
					#pCity = pyCity.GetCy()
					#pHyboremPlayer.acquireCity(pCity,false,false)
					#pPlot = cf.findClearPlot(-1, gc.getMap().plot(pCity.getX(),pCity.getY()))
					#iUnit = gc.getInfoTypeForString('UNIT_MINOTAUR')
					#newUnit = pHyboremPlayer.initUnit(iUnit, pPlot.getX(), pPlot.getY(), UnitAITypes.UNITAI_CITY_DEFENSE, DirectionTypes.DIRECTION_SOUTH)
		if self.isFourthACLevelReached():
			self.setFourthACLevelReached()
			#TODO Give flight to Hyborem's units or make sure they can attack overseas...

